@@ -36,6 +36,8 @@
  *                       and updated to GPL v3
  * 09 Nov 08, arkadiusz.firus@gmail.com: adopt safe_printf
  * 07 Oct 11, jf@dockes.org, fix to add_to_collection()
+ * 09 Jun 13, peterli@salk.edu, added code to cope with bold/italic
+ 			requests that are not properly nested
  *--------------------------------------------------------------------*/
 
 #ifdef HAVE_CONFIG_H
@@ -402,7 +404,7 @@ attrstack_copy_all (AttrStack *src, AttrStack *dest)
  * Name:	attrstack_unexpress_all
  * Purpose:	Routine to un-express all attributes heretofore applied,
  * 		without removing any from the stack.
- * Args:	Stack whost contents should be unexpressed.
+ * Args:	Stack whose contents should be unexpressed.
  * Returns:	None.
  * Notes:	This is needed by attrstack_push, but also for \cell, which
  * 		often occurs within a brace group, yet HTML uses <td></td> 
@@ -862,3 +864,57 @@ free_collection(Collection *c)
 	}
 }
 
+#define SUPPORT_UNNESTED
+#ifdef SUPPORT_UNNESTED
+
+// Push onto AttrStack generally, without extra bells and whistles
+void attr_push_gen(AttrStack *stack, int attr, char *param) {
+  stack->tos++;
+  stack->attr_stack[stack->tos] = attr;
+  if (param)
+    stack->attr_stack_params[stack->tos] = param;
+  else
+    stack->attr_stack_params[stack->tos] = NULL;
+}
+
+// Pop from AttrStack generally, without extra bells and whistles
+void attr_pop_gen(AttrStack *stack, int *attr, char **param) {
+  *attr = stack->attr_stack[stack->tos];
+  *param = stack->attr_stack_params[stack->tos];
+  stack->tos--;
+}
+
+// Pop attrs until we find the match, then push the non-matches back
+static AttrStack temp_stack;
+int attr_find_pop(int findattr) {
+  temp_stack.tos = -1; // initialize temp_stack if necessary
+
+  AttrStack *stack = stack_of_stacks_top;
+  if (!stack) {
+    warning_handler ("no stack to pop attributes from");
+    return FALSE;
+  }
+
+  int attr = ATTR_NONE;
+  char *param = NULL;
+
+  // Look for findattr on stack, expressing endings.
+  // Store any non-matches to temp_stack
+  while (stack->tos >= 0) {
+    attr_pop_gen(stack, &attr, &param);
+    attr_express_end(attr, param);
+    if (attr == findattr) break;
+    attr_push_gen(&temp_stack, attr, param);
+  }
+
+  // Put the non-matches back, expressing beginnings
+  while (temp_stack.tos >= 0) {
+    attr_pop_gen(&temp_stack, &attr, &param);
+    attr_push(attr, param);
+  }
+
+  return TRUE;
+}
+#endif /* SUPPORT_UNNESTED */
+
+
