@@ -31,6 +31,7 @@
  * 29 Mar 05, daved@physiol.usyd.edu.au: changes requested by ZT Smith
  * 16 Dec 07, daved@physiol.usyd.edu.au: updated to GPL v3
  * 09 Nov 08, arkadiusz.firus@gmail.com: added leave_line
+ * 25 Sep 11, jf@dockes.org: added allocmem()
  *--------------------------------------------------------------------*/
 
 #ifdef HAVE_CONFIG_H
@@ -48,14 +49,11 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 
-#ifdef HAVE_STDIO_H
-#include <stdio.h>
-#endif
-
-
 #include "malloc.h"
 #endif
 
+#include "util.h"
+
 /*========================================================================
  * Name:	h2toi
  * Purpose:	Converts a 2-digit hexadecimal value to an unsigned integer.
@@ -110,9 +108,58 @@ concatenate (const char *s1, const char *s2)
 {
 	char *result;
 
-	result = my_malloc((strlen(s1) + strlen(s2) + 1) * sizeof(char));
+	if ((result = my_malloc((strlen(s1) + strlen(s2) + 1) * sizeof(char))) == 0)
+        	return 0;
 	strcpy(result, s1);
 	strcat(result, s2);
 
 	return result;
 }
+
+/*========================================================================
+ * Name:	allocmem
+ * Purpose:	allocate or reallocate memory
+ * Args:	array, size, current allocation, min allocation, max increment
+ * Returns:	pointer to allocated array
+ *=======================================================================*/
+
+/*
+ * Note:  This is designed for programs that use contiguous blocks of
+ * increasing size. The problem when using a simple realloc is that a lot of
+ * memory is lost to fragmentation if many fixed size reallocations are 
+ * performed.
+ * This routine doubles the memory space every time it is called, 
+ * Algorithm: 
+ * - 1st call (cp = 0), allocate min *sz
+ * - next calls double current size
+ * - Until current size is maxinc * sz, then increment by maxinc*sz everytime
+ */
+void *
+allocmem
+(
+	void 	*cp,	/* The array to grow. may be NULL */
+	int	 sz,	/* Unit size in bytes */
+	int 	*np,    /* Pointer to current allocation number */
+	int	 min,   /* Number to allocate the first time */
+	int	 maxinc /* Maximum increment */
+)
+{
+  int inc;
+
+  if (cp == NULL) {
+    cp = malloc(min * sz);
+    if (cp)
+      *np = min;
+    else
+      *np = 0;
+    return(cp);
+  }
+  if (*np > maxinc)
+    inc = maxinc;
+  else
+    inc = *np;
+  cp = realloc(cp, (*np + inc) * sz);
+  if (cp)
+    *np += inc;
+  return cp;
+}
